#!/usr/bin/env python

#****************************************************************************
#** $Id: sqltable4.py,v 1.0 2002/06/15 17:33:53 frispete Exp $
#**
#** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
#**
#** This file is part of an example program for PyQt.  This example
#** program may be used, distributed and modified without limitation.
#**
#*****************************************************************************/

import sys
from qt import *
from qtsql import *

TRUE  = 1
FALSE = 0

#DB_QTDRIVER = "QPSQL7"
DB_QTDRIVER = "QMYSQL3"
DB_HOSTNAME = "server"
DB_DATABASE = "testdb"
DB_USERNAME = "xy"
DB_PASSWORD = "zzz"

class CustomTable(QDataTable):
    def __init__(self, cursor, autoPopulate = FALSE, parent = None, name = None):
        QDataTable.__init__(self, cursor, autoPopulate, parent, name)

    def paintField(self, p, field, cr, b):
        if not field:
            return
        if str(field.name()) == "statusid":
            query = QSqlQuery("SELECT name FROM status WHERE id=%s" %
                              field.value().toString())
            text = ""
            if query.next():
                text = query.value(0).toString()
            p.drawText(2, 2, cr.width()-4, cr.height()-4,
                        self.fieldAlignment(field), text)
        else:
            QDataTable.paintField(self, p, field, cr, b)


class StatusPicker(QComboBox):
    def __init__(self, parent = None, name = None):
        print "StatusPicker.__init__"
        QComboBox.__init__(self, parent, name)
        self.index2id = {}
        cur = QSqlCursor("status")
        cur.select(cur.index("name"))
        i = 0
        while cur.next():
            print "SP:", cur.value("name").toString(), cur.value("id").toInt()
            self.insertItem(cur.value("name").toString(), i)
            self.index2id[i] = cur.value("id").toInt()
            i += 1

    def statusId(self):
        id = None
        i = self.currentItem()
        if self.index2id.has_key(i):
            id = self.index2id[i]
        print "statusId:", id, self.index2id.items()
        return id

    def setStatusId(self, statusid):
        print "setStatusId:", statusid
        for i, j in self.index2id.items():
            if j == int(statusid):
                self.setCurrentItem(i)
                break

class CustomSqlEditorFactory(QSqlEditorFactory):
    def __init__(self):
        print "CustomSqlEditorFactory.__init__"
        QSqlEditorFactory.__init__(self)

    def createEditor(self, parent, field):
        print "createEditor:", self, parent, field
        try:
            if str(field.name()) == "statusid":
                return StatusPicker(parent)
        except AttributeError:
            pass
        return QSqlEditorFactory.createEditor(self, parent, field)

class Table(CustomTable):
    def __init__(self):
        self.staffCursor = QSqlCursor("staff")
        QDataTable.__init__(self, self.staffCursor)
        self.propMap = QSqlPropertyMap()
        self.editorFactory = CustomSqlEditorFactory()
        self.propMap.insert("StatusPicker", "statusid")
        self.installPropertyMap(self.propMap)
        self.installEditorFactory(self.editorFactory)

        self.addColumn("forename", "Forename")
        self.addColumn("surname",  "Surname" )
        self.addColumn("salary",   "Annual Salary")
        self.addColumn("statusid", "Status")
        self.order = QStringList("surname")
        self.order.append("forename")
        self.setSort(self.order)
        self.refresh()


def createConnection():
    db = QSqlDatabase.addDatabase(DB_QTDRIVER)
    if db:
        db.setHostName(DB_HOSTNAME)
        db.setDatabaseName(DB_DATABASE)
        db.setUserName(DB_USERNAME)
        db.setPassword(DB_PASSWORD)
        if db.open():
            return TRUE
        else:
            print "Could not open testdb database"
            print db.lastError().driverText()
            print db.lastError().databaseText()
    else:
        print "Could not activate database driver"
    return FALSE

if __name__=='__main__':
    app = QApplication( sys.argv )

    if createConnection():
        t = Table()
        app.setMainWidget(t)
        t.show()
        app.exec_loop()


